@@ -49,6 +49,13 @@ class Configuration(BaseModel):
         validate_assignment = True
 
 
+class git(BaseModel):
+    repo: str
+    host: str = "github.com"
+    branch: str = "main"
+    commit: Optional[str] = None
+
+
 class Version(PkgVersion):
     def __new__(cls, *args, **kwargs):
         if args and isinstance(args[0], Version):
